#source("S1_soilInterpolation.r")
source("lib.r")

datDir    <- lib.datDir
plotDir   <- lib.plotDir

VWC_probeDepths <- c(10,20,40,60)
SWIP_probeDepths <- c(10,20,40,60,80,100)

pngResolution <- 300

#================================================

timeRange <- as.POSIXct(c("2019-05-15 00:00","2019-08-08 12:00"))
dat <- data.frame(time = seq(timeRange[1], timeRange[2], 3600))

#========== initialize soil array ============================


soilArray <- array(NA, dim=c(nrow(lib.hourly), 200, 3))

dimnames(soilArray)[[3]] <- c("VWC", "delta18O", "deltaD")

soilArray[,ncol(soilArray),"VWC"] <- 30	
soilArray[,ncol(soilArray),"delta18O"] <- -10.5
soilArray[,ncol(soilArray),"deltaD"] <- -73

#============SOIL MOISTURE DATA=================================================================================

cat("Load and interpolate soil mositure data...\n")

soil   <- read.table(paste0(datDir,"/soilMoist.dat.gz"), sep=',', skip=4)
names(soil) <- c("time", "record", "Tpanel", "BattVolt", sprintf("VWC%02d",1:16), sprintf("Permit%02d",1:16), sprintf("Temp%02d",1:16))
soil$time <- as.POSIXct(soil$time)
soil <- soil[soil$time >= as.POSIXct("2019-05-10"),]

VWC <- data.frame(time=soil$time, d10 = apply(soil[,c(0,4)+5],1,median),
                                  d20 = apply(soil[,c(0,4)+6],1,median),
								  d40 = apply(soil[,c(0,4)+7],1,median),
								  d60 = apply(soil[,c(0,4)+8],1,median))								  								 
#----------------
								  
vwcProbes <- list()
	
for(d in c(VWC_probeDepths)){
	dDat <-VWC[,paste0("d",d)]
	x <- aggregate(VWC[,paste0("d",d)], list(time=as.POSIXct(format(VWC$time,"%Y-%m-%d %H:00"))),mean)	
	
	vwcProbes[[length(vwcProbes)+1]] <- list(depth=d, time=x$time)
	
	soilArray[,d,"VWC"] <- merge(x, lib.hourly[,"time",drop=FALSE], all.y=TRUE)$x
}

for(p in c("VWC")){

	#----- vertical interpolation -------
	for(n in 1:dim(soilArray)[1]){				
			valMask <- is.finite(soilArray[n,,p])
			if(sum(valMask) == ncol(soilArray) | sum(valMask) <2) next	
			soilArray[n, ,p] <- approx(which(valMask), soilArray[n, valMask,p], 1:ncol(soilArray), rule=2)$y			
	}

	#----- horizontal interpolation -----
	for(n in 1:dim(soilArray)[2]){
		valMask <- is.finite(soilArray[,n,p])
		soilArray[,n,p] <- round(approx(which(valMask), soilArray[valMask,n,p], 1:dim(soilArray)[1], rule=2)$y,2)
	}  
}												 

#============SOIL ISOTOPE DATA=================================================================================
dataList <- list(cores=list(), probes=list())

#------- fill in soil core data ---------

cat("Load soil isotope data from soil cores...\n")

coreDat <- read.table(paste0(datDir,"/soilCores.csv"), sep='\t', header=TRUE)
coreDat$time <- as.POSIXct(coreDat$time)
IDs    <- c(1,2,4)
before <- c(6, 6, 6)
after  <- c(6, 100, 6)
for(n in 1:length(IDs)){
	id <- IDs[n]
	sel <- coreDat[coreDat$ID==id,]
	coreTimeIndex <- which(lib.hourly$time == sel$time[1])
	
	dataList$cores[[length(dataList$cores)+1]] <- list(time=lib.hourly$time[coreTimeIndex],depth=range(c(sel$fromDepth, sel$toDepth)))
	
	coreTimeMask <- coreTimeIndex + c(-before[n]:after[n])
	for(n in nrow(sel):1){
		soilArray[coreTimeMask, sel$toDepth[n]:1,"delta18O"] <- sel$delta18O[n]
		soilArray[coreTimeMask, sel$toDepth[n]:1,"deltaD"] <- sel$deltaD[n]
	}
	
	valMask <- c(1:max(sel$toDepth), ncol(soilArray))
	
	soilArray[coreTimeIndex, ,"delta18O"] <- approx((1:ncol(soilArray))[valMask], soilArray[coreTimeIndex, valMask,"delta18O"], 1:ncol(soilArray))$y
	soilArray[coreTimeIndex, ,"deltaD"] <- approx((1:ncol(soilArray))[valMask], soilArray[coreTimeIndex, valMask,"deltaD"], 1:ncol(soilArray))$y
	
	for(i in coreTimeMask){
		soilArray[i, ,"delta18O"] <- soilArray[coreTimeIndex, ,"delta18O"]
		soilArray[i, ,"deltaD"] <- soilArray[coreTimeIndex, ,"deltaD"]
	}
	
}

#-------fill in in-situ probe data -------

cat("Load soil isotope data from SWIP measurements...\n")

probeDat <- read.table(paste0(datDir,"/WIP_liquidFixed.csv"), sep='\t', header=TRUE)
probeDat$time <- as.POSIXct(probeDat$time)
	
for(n in 1:length(SWIP_probeDepths)){

	depthIndex <- SWIP_probeDepths[n]

	sel <- probeDat[probeDat$WIP == sprintf("S%dcm",SWIP_probeDepths[n]),]
	
	# exclude data from periods with measurements problems
	for(i in 1:nrow(lib.maskRanges)){
		sel <- sel[sel$time < lib.maskRanges[i,1] | sel$time > lib.maskRanges[i,2],]
	}
	
	# remember times of avaiable measurements
	dataList$probes[[length(dataList$probes)+1]] <- list(depth=depthIndex, time=sel$time)
	
	for(isotope in c("delta18O", "deltaD")){
		soilArray[,depthIndex,isotope] <- lib.rollmedian(approx(sel$time, sel[,isotope], xout=lib.hourly$time)$y,31)
	}
}

#----- vertical interpolation -------
cat("Interpolate soil isotope data vertically...\n")

for(n in 1:dim(soilArray)[1]){
	valMask <- is.finite(soilArray[n,,"delta18O"])
	if(sum(valMask) == ncol(soilArray) | sum(valMask) <2) next
	
	soilArray[n, ,"delta18O"] <- approx(which(valMask), soilArray[n, valMask,"delta18O"], 1:ncol(soilArray), rule=2)$y
	soilArray[n, ,"deltaD"] <- approx(which(valMask), soilArray[n, valMask,"deltaD"], 1:ncol(soilArray), rule=2)$y
}

#----- horizontal interpolation -----
cat("Interpolate soil isotope data horizontally...\n")

for(n in 1:dim(soilArray)[2]){
	valMask <- is.finite(soilArray[,n,1])
	soilArray[,n,"delta18O"] <- approx(which(valMask), soilArray[valMask,n,"delta18O"], 1:dim(soilArray)[1], rule=2)$y
	soilArray[,n,"deltaD"] <- approx(which(valMask), soilArray[valMask,n,"deltaD"], 1:dim(soilArray)[1], rule=2)$y	
}

#--------- write interpolated soilArray to RDA file -------------------
#saveRDS(soilArray, file=paste0(datDir,"/derivedData/soilArray.Rda"))

dimnames(soilArray)[[1]] <- format(lib.hourly$time, "%Y-%m-%d %H:00")
dimnames(soilArray)[[2]] <- sprintf("%03d cm",1:ncol(soilArray))

precision <- c(VWC=1, delta18O=2, deltaD=1)

for(p in dimnames(soilArray)[[3]]){
	cat("Write interpolated", p, "data to file...\n")
	gz = gzfile(paste0(datDir,"/derivedData/S1_soilArray_",p,".csv.gz"), 'w')
	write.table(round(soilArray[,,p],precision[p]), file=gz, quote=FALSE, sep=',', col.names=NA)
	close(gz)
}

#=====================================================================================

#------ plot soilArray -------
cat("Plot interpolated soil data...\n")

ranges <- list(VWC=c(6,46,2),			  
			   delta18O=c(-11.5,-3.5,0.5),
			   deltaD=c(-75,25,5),
			   DEXcess=c(-5,115,10))
				   
subFigLabels <- list(VWC="a", delta18O="b", deltaD="c")

colPalettes <- list(VWC="BuPu",					
					delta18O="YlOrRd",
					deltaD="YlOrRd")

png(paste0(plotDir,"/S1_interpolatedSoilData.png"), width=lib.singleWidth, height=5, units='in', res=pngResolution)
#pdf(paste0(plotDir,"/S1_interpolatedSoilData.pdf"), width=lib.singleWidth, height=5)

	layout(cbind(c(1,3,5,7),c(2,4,6,8)), widths=c(10,2.5), heights=c(5,5,5,0.8))	
	
	for(p in c("VWC", "delta18O", "deltaD")){

		breaks <- matrix(seq(ranges[[p]][1],ranges[[p]][2], by=0.5*ranges[[p]][3]))
		centers <- matrix(seq(ranges[[p]][1] + ranges[[p]][3]/2,ranges[[p]][2]-ranges[[p]][3]/2, length.out=length(breaks)-1))
		cols <- hcl.colors(length(centers), colPalettes[[p]], rev = TRUE)

		par(mar=c(0.1,3,0.05,1),mgp=c(2,0.5,0),cex=0.75)
		image(lib.hourly$time, seq(0, ncol(soilArray), 1), soilArray[,,p], ylim=c(125,-5),
			  xlab='', ylab="Depth[cm]", las=1, zlim=ranges[[p]][1:2],bty='n',xaxt='n',
			  col=cols, breaks=breaks)		
		
		abline(v=as.POSIXct(c("2019-05-21 08:00","2019-05-21 18:00")),lty=2, lwd=0.75)
		xVals <- as.POSIXct(c("2019-06-01","2019-05-22"))
		arrows(xVals[1], 115, xVals[2], 115, len=0.075)
		text(xVals[1], 115, labels="irrigation", adj=c(-0.1, 0.5), cex=1)
		
		if(p == "VWC"){
			for(probe in vwcProbes){
			tDiffs <- diff(as.numeric(probe$time))
			tGaps <- which(tDiffs > 0.5*86400)+1
			tStarts <- c(probe$time[1], probe$time[tGaps])
			tEnds <- c(probe$time[tGaps-1], tail(probe$time,1))
			rect(tStarts, probe$depth-2, tEnds, probe$depth+2)
		}	
		
		}else{
		
			for(core in dataList$cores){
				rect(core$time - 43200, max(core$depth), core$time + 43200, min(core$depth))
			}

			for(probe in dataList$probes){					
				tDiffs <- diff(as.numeric(probe$time))
				tGaps <- which(tDiffs > 1.5*86400)+1
				tStarts <- c(probe$time[1], probe$time[tGaps])
				tEnds <- c(probe$time[tGaps-1], tail(probe$time,1))
				shortMask <- tStarts == tEnds
				tStarts[shortMask] <- tStarts[shortMask] - 28800
				tEnds[shortMask] <- tEnds[shortMask] + 28800
				o <- (probe$depth-100)*+0.01
				rect(tStarts, probe$depth-2+o, tEnds, probe$depth+2+o)
				
			}
		}

		lib.label_subPlot(subFigLabels[[p]], fx=-0.15)
		
		par(mar=c(1,0,2.5,0.5))			
		image(c(1,2,3,4), breaks, t(cbind(centers,NA,NA,NA)), axes=FALSE, xlab='', ylab='',col = cols, breaks=breaks, main=lib.labelList[[p]])
		
		breaks <- matrix(seq(ranges[[p]][1],ranges[[p]][2], by=3*ranges[[p]][3]))
		axis(4, at= breaks, labels = FALSE, line=-3)
		text(3, breaks, labels = breaks, xpd=TRUE,cex=1)
		
	}
	
	par(mar=c(0,3,0,1))
	plot(lib.hourly$time, lib.hourly$time, ylim=0:1, axes=FALSE, xaxs='i', ylab='')
	
	tickTimes <- as.POSIXct("2019-05-15") + seq(0:100)*86400
	axis.POSIXct(3, lib.hourly$time, at= tickTimes, labels=FALSE,tcl=0.25, mgp=c(-2,-2.5,0))
	
	tickTimes <- as.POSIXct(sprintf("2019-%02d-%02d", c(5,6,7,8),c(15,15,15,8)))
	axis.POSIXct(3, lib.hourly$time, at= tickTimes, format="%d.%b",tcl=0.5, mgp=c(-2,-1.5,0))
	
dev.off()
