#source("S2_XWIPs_and_RWU.r")
source("lib.r")
source("wut.r")

datDir    <- lib.datDir
plotDir   <- lib.plotDir


#========== load interpolated soil data (see S1_soilInterpolation.r) into an array ==========
soilArray <- NULL
parameters <- c("VWC","delta18O","deltaD")
for(i in 1:length(parameters)){
	p <- parameters[i]
	x <- read.table(paste0(datDir, "/derivedData/S1_soilArray_", p, ".csv.gz"), sep=",",header=TRUE)
	arrayTimes <- as.POSIXct(x[,1])
	x<-as.matrix(x[,-1])

	if(is.null(soilArray))
		soilArray <- x
	else 
		soilArray <- array(c(c(soilArray), x), dim = c(dim(soilArray)[1], dim(soilArray)[2], i))
}
dimnames(soilArray) <- list(rownames(x), colnames(x), parameters)

#========== load XWIP data and compute daily median values for each probe ===================

probeDat <- read.table(paste0(datDir,"/WIP_liquidFixed.csv"), sep='\t', header=TRUE)
probeDat$time <- as.POSIXct(probeDat$time)
probeDat <- probeDat[probeDat$time < as.POSIXct("2019-08-08"),]

xylDat <- NULL
for(id in unique(probeDat$WIP)){
	if(!length(grep('^T',id))) next
	sel <- probeDat[probeDat$WIP==id,]
	if(id=="T44R") sel <- sel[sel$time < as.POSIXct("2019-07-05"),]	
	aggTime <- as.POSIXct(format(sel$time,"%Y-%m-%d"))	
	aggSel <- aggregate(sel[,c("delta18O", "deltaD")], list(time=aggTime),function(x) round(median(x),2))
	xylDat <- rbind(xylDat, data.frame(aggSel, WIP = id))
}

#============ aggregate XWIP data per installation height ==============================================
xylDat$time <- xylDat$time + 43200 # shift time towards center of day

aggXylDat <- aggregate(xylDat[,c("delta18O","deltaD")], list(time=xylDat$time, WIP = gsub("T.","",xylDat$WIP)), mean)
maxXylDat <- aggregate(xylDat[,c("delta18O","deltaD")], list(time=xylDat$time, WIP = gsub("T.","",xylDat$WIP)), max)
minXylDat <- aggregate(xylDat[,c("delta18O","deltaD")], list(time=xylDat$time, WIP = gsub("T.","",xylDat$WIP)), min)

# mask to discard values of unreliable periods
mask <- aggXylDat$time < lib.maskRanges[3,1] | aggXylDat$time > lib.maskRanges[3,2]

aggXylDat <- aggXylDat[mask,] # daily mean values across all XWIPs at one hieght
maxXylDat <- maxXylDat[mask,] # daily max values across all XWIPs at one hieght
minXylDat <- minXylDat[mask,] # daily min values across all XWIPs at one hieght

# write to file
aggXylDat[,c("delta18O","deltaD")] <- round(aggXylDat[,c("delta18O","deltaD")],2)
write.table(aggXylDat, paste0(datDir,"/derivedData/S2_aggXylDat.csv"), sep=',', row.names=FALSE, quote=FALSE)

#======= compute RWU signatures for different rooting depths =======
# depths levels of the soilArray (in cm)
z        <- 1:ncol(soilArray)

# RWU model parameters
theta_s  <- 0.45
theta_w  <- 0.08
theta_c1 <- 0.5
theta_c2 <- 1

# RWU signature for zr value of 90 cm
rootDist <- wut.generate_vertRootDist_GerwitzPage(z, zr=90)
RWU_90   <- wut.get_RWU_signature(soilArray, rootDist, z, theta_s, theta_w, theta_c1, theta_c2)

# RWU signature for zr value of 60 cm
rootDist <- wut.generate_vertRootDist_GerwitzPage(z, zr=60)
RWU_60 <- wut.get_RWU_signature(soilArray, rootDist, z, theta_s, theta_w, theta_c1, theta_c2)

# RWU signature for zr value of 120 cm
rootDist <- wut.generate_vertRootDist_GerwitzPage(z, zr=120)
RWU_120 <- wut.get_RWU_signature(soilArray, rootDist, z, theta_s, theta_w, theta_c1, theta_c2)

#==== write RWU of the model with zr=90cm to a text file ==============

# round results to two decimal places
RWU_90[,-1] <- round(RWU_90[,-1],2) 

# write to file
write.table(RWU_90, paste0(datDir,"/derivedData/S2_RWU.csv"), sep=',', row.names=FALSE, quote=FALSE)

#======= plot measured XWIP signatures and modelled RWU sigantures ===

modelCol="grey75"
pointOpacity="aa"

pdf(paste0(plotDir,"/S2_XWIP_vs_RWU.pdf"), width=lib.singleWidth, height=2)
	layout(matrix(c(2,1,3), nrow=3, byrow=TRUE), heights=c(5,5,1))
	par(mar=c(0.05,3,0.1,0.2), tcl=0.5, mgp=c(1.6,0.2,0), cex=0.75, las=1)
	
	for(isotope in c("deltaD", "delta18O")){
		plot(RWU_90$time, RWU_90[,isotope], col=modelCol, type='l',xlim=as.POSIXct(c("2019-05-17","2019-08-06")),
			 ylim=list(delta18O=c(-10,-4), deltaD=c(-70,10))[[isotope]], lwd=2, ylab=lib.labelList[[isotope]], xaxt='n')
		axis.POSIXct(1, lib.hourly$time, labels=isotope=="deltaD")
			 
		lines(RWU_60$time, RWU_60[,isotope], col=modelCol)
		lines(RWU_120$time, RWU_120[,isotope], col=modelCol)
					 
		for(WIP in c("B","H","R")){		
			mask <- aggXylDat$WIP == WIP
			col <- paste0(lib.colRBH[match(WIP,c("R","B","H"))], pointOpacity)
					
			pch <- rep(20, sum(mask))
			cex <- rep(0.6, sum(mask))
			noRangeMask <- maxXylDat[mask,isotope]==minXylDat[mask,isotope]
			pch[noRangeMask] <- 1
			cex[noRangeMask] <- 0.4
					
			o <- c(R=-3600,B=0,H=3600)[WIP]*3
			points(aggXylDat$time[mask]+o, aggXylDat[mask,isotope], pch=pch, cex=cex, col = col)		
			arrows(aggXylDat$time[mask]+o, minXylDat[mask,isotope], aggXylDat$time[mask]+o, maxXylDat[mask,isotope], length=0, col=col)
		}	
		
		lib.label_subPlot(c("a","b")[1+(isotope=="deltaD")],TRUE, fx=0.05)			
		
		legend(as.POSIXct("2019-05-16"), -3.3, col=c(modelCol,paste0(lib.colRBH, pointOpacity)), lty=c(1,NA,NA,NA), lwd=c(2,1,1,1), pch=c(NA, 20, 20, 20), 
				   legend=c(expression(delta[RWU]~"(model)"), expression(delta[xyl.R]~"(0.1 m)"), expression(delta[xyl.B]~"(1.5 m)"), expression(delta[xyl.H]~"(8.0 m)")), cex=0.8, bty='n')
				   
		text(as.POSIXct("2019-07-07 12:00"), c(-6.6,-7.55,-8.5), labels=c(expression(z[r]*"=0.6 m"),expression(z[r]*"=0.9 m"),expression(z[r]*"=1.2 m")), cex=0.5)
			
	}
dev.off()

