#source("S3_transformation_and_convolution.r")
source("lib.r")
source("wut.r")

# load sap flow velocities
forcing <- read.table(paste0(lib.datDir, "/climAndSapDat.csv"), sep='\t', header=TRUE)
forcing$time <- as.POSIXct(forcing$time)
forcing$vSap <- forcing$vSap/24/1000 # convert hourly vasp values from mm/d to m/h

# load xylem signatures as aggregated from "S2_XWIPs_and_RWU.r"
xyl <- read.table(paste0(lib.datDir,"/derivedData/S2_aggXylDat.csv"), sep=',', header=TRUE)
xyl$time <- as.POSIXct(xyl$time)

# load RWU signatures as computed from "S2_XWIPs_and_RWU.r"
RWU <- read.table(paste0(lib.datDir,"/derivedData/S2_RWU.csv"), sep=',', header=TRUE)
RWU$time <- as.POSIXct(RWU$time)

#=== compute cumulative sap flow distances for times of interest=================

# starting point of the irrigation (will be regarded as 0 m)
startDistance <- wut.time2length(inTime = forcing$time, inLength = cumsum(forcing$vSap), outTime = as.POSIXct("2019-05-21 12:00"))

# cumulative sap flow distances at all times where RWU signatures are available
RWU$distance <- wut.time2length(inTime = forcing$time, inLength = cumsum(forcing$vSap), outTime = RWU$time)

# cumulative sap flow distances at all times where xylem water signatures are available
xyl$distance <- round(wut.time2length(inTime = forcing$time, inLength = cumsum(forcing$vSap), outTime = xyl$time),2)

write.table(RWU, paste0(lib.datDir,"/derivedData/S3_RWU_with_sapDistance.csv"), sep=',', row.names=FALSE, quote=FALSE)
write.table(xyl, paste0(lib.datDir,"/derivedData/S3_aggXylDat_with_sapDistance.csv"), sep=',', row.names=FALSE, quote=FALSE)

#==== transform modelled RWU by convolution with transfer functions =============

# create an equally spaced vector of sap flow distances
d <- seq(0, 20, 0.1)

R <- xyl[xyl$WIP=="R",] # get observed xylem signatures at 0.1 m
H  <- xyl[xyl$WIP=="H",] # get observed xylem signatures at 8 m

# create FPLDs based on fitted F-Distributions
hF1 <- data.frame(x=d,  y = wut.FDist(c(shape=3.4, scale=1.43, lag=0), d))
hF2  <- data.frame(x=d, y = wut.FDist(c(shape=0.62, scale=1.26, lag=1.64), d))

# obtain hF1F2 by convolving hF1 with hF2
hF1F2 <- data.frame(x=d, y = wut.convolve(hF1$y, hF2$y))

# transform modelled RWU signatures with transfer functions hR and hRH
# the function "wut.convolve_RWU_signature"
#  - transforms time series to the sap flow distance domain
#  - convolves the sap flow distance series of modelled RWU signatures with a transfer function
#  - transforms the resulting sap flow distance series back to a time series
RWU_R <- wut.transfer_signature(RWU[,c("deltaD","delta18O")], RWU$time, RWU$distance, hF1)
RWU_H  <- wut.transfer_signature(RWU[,c("deltaD","delta18O")], RWU$time, RWU$distance, hF1F2)


#--------- plot modelled RWU signatures and modelled and observed RWU and xylem signatures -----------
modelCol <- "grey50"
pdf(paste0(lib.plotDir,"/S3_convolve_RWU_with_FPLD.pdf"), width=lib.singleWidth, height=2)

	par(mar=c(1.75,3,0.1,0.3), tcl=0.4, mgp=c(1.8,0.2,0), cex=lib.cex, las=1)
	
	isotope="delta18O"
		
		# plot modelled RWU signature
		plot(RWU$time, RWU[,"deltaD"], lwd=2, col=modelCol, type='l', xaxt='n', ylab=lib.labelList[["deltaD"]], xlim = RWU$time[c(1,30)])
		axis.POSIXct(1, RWU$time, at=as.POSIXct(sprintf("2019-%d-%d 00:00",c(5,5,6,6,6),c(20,26,1,7,13))))
		
		# plot observed xylem signatures
		points(R$time, R[,"deltaD"], col=lib.colRBH["R"], pch=0, cex=0.6)
		points(H$time, H[,"deltaD"], col=lib.colRBH["H"], pch=0, cex=0.6)
		
		# plot predicted xylem signatures
		lines(RWU$time, RWU_R[,"deltaD"], col=lib.colRBH["R"], lwd=1.5)
		lines(RWU$time, RWU_H[,"deltaD"], col=lib.colRBH["H"], lwd=1.5)
		
		legend("topleft", col=c(modelCol,lib.colRBH[c("R","H")]), lwd=c(2, 1, 1), lty=c(1,NA,NA), pch=c(NA, 0, 0), 
				   legend=c(expression(delta[RWU]~"(model)"), expression(delta[xyl.R]~"(0.1 m)"), expression(delta[xyl.H]~"(8.0 m)")), cex=0.7, bty='n')				
	
	# add subfigure that shows the shape of the FPLDs (=transfer functions)
	par(fig=c(0.55,0.97,0.15,0.7), mar=c(1.8,1.8,0,0), mgp=c(0,-0.35,0), new=TRUE, cex=lib.cex*0.6)
	plot(wut.unity(hF1), lwd=1.5, yaxt='n', ylab="Density", type='l', col=lib.colRBH["R"], xlim=c(0,10), xlab="Sap flow distance [m]")
	lines(wut.unity(hF1F2), lwd=1.5, col=lib.colRBH["H"])
	lines(wut.unity(hF2), lwd=1, col="white")
	lines(wut.unity(hF2), lwd=0.7, col=lib.colRBH["H"])
	
	legend("topright", lwd=c(1.5,0.7,1.5), col=lib.colRBH[c("R","H","H")], legend=c(expression(h[F1]),expression(h[F2]),expression(h[F1F2])),bty='n', x.intersp=0.3)
	
dev.off()
