#source("S4_xylem_water_age.r")
source("lib.r")
source("wut.r") # "library" containing functions for signal transformation and convolution


#====  data preparation ==============================

# ----- load daily sap flow data ------
# load extended daily sap flow data into a buffer data frame
buff <- read.table("../Data/extended_sapFlow_daily.csv", sep='\t', header=TRUE)
buff$day <- as.Date(buff$day)

# prepare a data frame to hold the final result (starting at a date in the previous growing season)
datD <- data.frame(day=as.Date("2018-08-01")+0:700, vSap=0)
	
# determine julian days for buffer and final data frame
jDay_buff <- format(buff$day,"%j")
jDay <- format(datD$day,"%j")

# copy sap flow values of the buffer data frame to each day of the final data frame with the same julian day
for(i in 1:length(jDay_buff)) datD$vSap[jDay==jDay_buff[i]] <- buff$vSap[i]

# compute cumulative sap flow distance, divide by 1000 to convert mm/d to m/d
datD$s <- cumsum(datD$vSap)/1000

# convert dates to POSIXct times, add offset of 12 hours to put values at mids of days
datD$time <- as.POSIXct(datD$day, tz='GMT')+12*3600


# ----- load hourly sap flow data ------
datH <- read.table("../Data/extended_sapFlow_hourly.csv", sep='\t', header=TRUE)

# compute cumulative sap flow distance, divide by 24000 to convert mm/d to m/h
datH$s <- cumsum(datH$vSap)/24000

# convert times to POSIXct, add offset of 1800 sesonds to put values at mids of hours
datH$time <- as.POSIXct(datH$time, tz='GMT')+1800
				  
#====  auxilliary functions ==============================

#---- functions to aggreagte the results ------------
# a function to extract age quantiles
get_ageQuantiles <- function(x){
	result <- NULL
	quants <- rev(c(0.01, 0.05,0.25, 0.5, 0.75, 0.95,0.99))
	for(q in quants){
		diffs <- abs(x-q)
		diffs[is.na(diffs)] <- Inf
		result <- c(result, which(diffs==min(diffs))[1])
	}
	names(result) <- sprintf("%.0f %%", quants*100)
	return(result-0.5)
}

# a function to extract age quantiles for each time step
get_statTable <- function(r){
	stat <- NULL
	for(n in 1:nrow(r)){
		vals <- r[n,][n:1]
		ageDist <- cumsum(vals) # <--- this is the distribution of water ages for time step n
		stat <- rbind(stat, get_ageQuantiles(ageDist)) # extract and store certain age quantiles
	}
	return(stat)
}

#========= computation of age percentiles ==========================================

# prepare equally spaced vector of sap flow distances for the FPLD
FPLD_maxLen <- 20   # FPLD maximum length [m]
FPLD_res    <- 0.1 # FPLD resolution [m]
s <- seq(0,FPLD_maxLen, FPLD_res)

# wut.FDist creates a parametric FPLD (see paper Eq. 21)
# parameters for hF1 and hF2 are the same as in paper Fig. 6c
hF1 <- wut.FDist(c(3.40,1.43, 0.00),s) #FPLD between RWU and stem base
hF2 <- wut.FDist(c(0.62,1.30, 1.64),s) #FPLD between 0.1 and 8 m stem height

hR <- hF1                    # use hF1 for the signal at 0.1 m stem height
hH <- wut.convolve(hF1,hF2)  # use hF1 convolved with hF2 for the signal at 8 m stem height

# define a list of scenarios (daily and hourly) and define some plot specific parameters for them
scenarios <- list(daily = list(xlim=as.POSIXct(c("2018-10-20","2019-11-10")), ylim=c(0,255),  dat = datD),
                  hourly = list(xlim=as.POSIXct(c("2019-05-05","2019-10-31")), ylim=c(0, 26), dat = datH))
				  

for(tempRes in names(scenarios)){

	cat("Compute xylem water ages at", tempRes, "resolution...\n")

	dat <- scenarios[[tempRes]]$dat

	# add a tiny offset which is linearly increasing to the cumulative sap flow distance
	# to avoid identical values during the dormant season (which would cause problems during transfomations)
	dat$s <- dat$s + + seq(0,0.1, length.out=nrow(dat))

	# create matrix to hold virtual tracer concentrations for each time step
	trac <- matrix(0, nrow=nrow(dat), ncol=nrow(dat))
	for(n in 1:nrow(dat)) trac[n,n]<-1

	# transform virtual tracer series with FPLD hR
	rR <- wut.transfer_signature(signature=trac, dat$time, dat$s, fpld=data.frame(x=s, y=hR))

	# transform virtual tracer series with FPLD hH
	rH <- wut.transfer_signature(signature=trac, dat$time, dat$s, fpld=data.frame(x=s, y=hH))
	
	# At this point, rR and rH hold matrices with concentartions of the virtual tracers for each timestep.
	# The water ages can now be derived from that virtual tracer matrix 
	
	# extract age quantiles from results
	scenarios[[tempRes]]$statR <- get_statTable(rR)
	scenarios[[tempRes]]$statH <- get_statTable(rH)
}


#========= plotting the results ==========================================

boxCol <- "#44aaff"

colH <- lib.colRBH["H"]
colR <- lib.colRBH["R"]
lwds <- c(0.5,1,1.5,2,1.5,1,0.5)


pdf(paste0(lib.plotDir,"/S4_xylemWaterAges.pdf"), width=lib.doubleWidth, height=4)
	layout(matrix(c(1,2)))
	par(mar=c(1.5,3,0.2,0.7), mgp=c(2,0.25,0), las=1, tcl=0.5, cex=lib.cex)

	plot(NA, xlim = scenarios$daily$xlim, ylim = scenarios$daily$ylim, xaxt='n', ylab="Xylem water age [d]")
	axis.POSIXct(1, x=scenarios$daily$dat$time)
	for(i in 1:7){		
		lines(scenarios$daily$dat$time, scenarios$daily$statH[,i], lwd=lwds[i], col=colH)
		lines(scenarios$daily$dat$time, scenarios$daily$statR[,i], lwd=lwds[i], col=colR)
	}
	rect(scenarios$hourly$xlim[1], 1, scenarios$hourly$xlim[2], scenarios$hourly$ylim[2], border=boxCol, lwd=2)
	
	legend("topright", lwd=lwds, col="#bbbbbb", legend=colnames(scenarios$daily$statH), title="Age percentiles")
	legend("topleft",  col=c(colH,colR), pch=15, legend=c("8 m","0.1 m"), title="   Stem height")


	plot(NA, xlim = scenarios$hourly$xlim, ylim = scenarios$hourly$ylim, xaxt='n', ylab="Xylem water age [d]")
	box(col=boxCol, lwd=2)
	axis.POSIXct(1, x=scenarios$hourly$dat$time)
	for(i in 1:7){
		lines(scenarios$hourly$dat$time, scenarios$hourly$statH[,i]/24, lwd=lwds[i], col=colH)
		lines(scenarios$hourly$dat$time, scenarios$hourly$statR[,i]/24, lwd=lwds[i], col=colR)
	}

	x <- scenarios$hourly$dat$time[1]+86400*2
	arrows(x, 0, x+86400*26, scenarios$hourly$ylim[2], length=0, lwd=4)
	text(x+86400*4, mean(scenarios$hourly$ylim)+5, labels="longer\nwarmup required",srt=45, cex=0.9)
dev.off()




