#(w)ater (u)ptake and (t)ransfer

# a helper function to normalize the input vector (or column named 'y', if input is a data frame)
#	this function ensures, that the FPLD has a total sum of 1
wut.unity <- function(x){
	if(class(x)=="numeric") return(x/sum(x))	
	x$y <- x$y/sum(x$y)
	return(x)
}

#==========================================================
# a function to generate vertical root density distributions
#	Gerwitz, A. and Page, E. R. (1974)
#	An Empirical Mathematical Model toDescribe Plant Root Systems
#	The Journal of Applied Ecology, 11, 773
#	https://doi.org/10.2307/2402227, https://www.jstor.org/stable/2402227?origin=crossref
wut.generate_vertRootDist_GerwitzPage <- function(zout, zr, f=3){	
	z <- 0:max(zr, max(zout))
	R <- f*1/zr * exp(-f*(z/zr))
	R <- approx(z, R, xout=zout)$y
	return(R/max(R))
}

#===================================================================
# parametric F-Distribution with additional lag to represent an FPLD
#  see paper Eq. 21
wut.FDist <- function(parameters, s){
	shape <- parameters[1]
	scale <- parameters[2]
	lag <- parameters[3]
	
	y <- df((s-lag)/scale, shape, 100)
	if(!is.finite(y[1])) y[1] <-0
	return(y)
}

#=========== RWU depth fractions and signatures ========================================================================
# in the following functions, soilArray is expected to be a 3D-Array with the follwoing dimensions:
# soilArray[depth, time, variable]
# the third dimension is expected to be:
#  VWC (volumetric water content)
#  delta18O (soil water delta18O)
#  deltaD (soil water deltaD)

# a function to predict the fractions of root water uptake over depth
#	Feddes, R. A., Kowalik, P., Kolinska-Malinka, K., and Zaradny, H. (1976)
#	Simulation of field water uptake by plants using a soil water de-pendent root extraction function
#	Journal of Hydrology, 31, 13–26
#	https://doi.org/10.1016/0022-1694(76)90017-2
wut.get_RWU_fractions <- function(soilArray, R, z, theta_s=0.45, theta_w=0.06, theta_c1=0.3, theta_c2=1, aggFormatString = "%Y-%m-%d 12:00"){
	R <- approx(z, R, xout= seq(1:ncol(soilArray)), rule=2)$y

	theta <- (soilArray[,,"VWC"]/100 - theta_w) / (theta_s-theta_w)
	alpha <- theta/theta_c1
	alpha[theta>theta_c1] <- 1
	
	RWU <- t(t(alpha)*R)
	return(RWU/rowSums(RWU))
}

# a function to compute RWU isotopic signatures (see paper Eq. 1)
wut.get_RWU_signature <- function(soilArray, R, z, theta_s=0.45, theta_w=0.06, theta_c1=0.3, theta_c2=1, aggFormatString = "%Y-%m-%d 12:00"){
	frac_RWU <- wut.get_RWU_fractions(soilArray, R, z, theta_s, theta_w, theta_c1, theta_c2)	
	r = data.frame(time = lib.hourly$time,
		           delta18O = rowSums(soilArray[,,"delta18O"] * frac_RWU),
	               deltaD = rowSums(soilArray[,,"deltaD"] * frac_RWU),
				   RWU_index = (frac_RWU[,10]+frac_RWU[,20])/(frac_RWU[,40]+frac_RWU[,60])
			       )
				   
	if(is.null(aggFormatString)) return(r)
	return(aggregate(r[,c("delta18O","deltaD","RWU_index")], list(time=as.POSIXct(format(r$time,aggFormatString))), function(x){median(x, na.rm=TRUE)}))
}

#========= transformation ==============================================
# see  paper Sec 2.1.3

# convert times to cumulative sap flow lengths
wut.time2length <- function(inTime, inLength, outTime){
	return(approx(inTime, inLength, outTime)$y)
}

# convert cumulative sap flow lengths to times
wut.length2time <- function(inLength, inTime, outLength){
	return(approx(inLength, inTime, outLength)$y)
}

# convolution of a tarcer series with a transfer function (=FPLD)
#  see paper Eq. 11
wut.convolve<-function(input, transferFun){
	inputLen <- length(input)	
	funLen   <- length(transferFun)
	buffLen  <- 2*max(inputLen,funLen)
	input     <- c(input,rep(0,buffLen-inputLen))
	transferFun <-c(transferFun,rep(0,buffLen-funLen))
	output<-Re(fft(fft(input)*fft(transferFun),inverse=TRUE))/buffLen
	return(output[1:inputLen])
}

# transformation of tracer time series
#  input time series to sap distance series, convolution with FPLD, output sap distance series to time series
wut.transfer_signature <- function(signature, times, lengths, fpld){
	
	# make sure the FPLD adds up to 1
	fpld    <- wut.unity(fpld)
	
	# infer spatial resolution for further computation from the passed FPLD
	res <- diff(fpld$x[1:2]) 
	
	fpld_values <- fpld$y
	
	# create regularly spaced length vector at the same resolution as fpld
	s <- seq(0, ceiling(max(lengths,na.rm=TRUE)/res)*res, res)
	
	# make sure signature is organized as matrix
	signature <- cbind(signature)
	
	result <- signature*NA
	
	for(n in 1:ncol(signature)){
		
		# resample RWU signature values to equally spaced lengths
		RWU <- approx(lengths, signature[,n], s, rule=2)$y
		
		# pad the RWU signature series with its first value
		padded_RWU <- c(rep(RWU[1], length(fpld_values)), RWU)
		
		# obtain concentration at root crown through convolution of RWU values with flow path length distribution
		padded_RC <- wut.convolve(padded_RWU, fpld_values)
		
		# remove padded data		
		RC <- padded_RC[-(1:length(fpld_values))]
		
		# resample data to the respective lengths of the original time steps
		result[,n] <- approx(s, RC, lengths)$y
	}
	
	return(result)
}